@@ -16,7 +16,10 @@
 ## 1: make
 make.bd.t <- function(tree, functions, sampling.f=NULL,
                       unresolved=NULL, control=list()) {
-  control <- modifyList(list(safe=FALSE, tol=1e-8, eps=0), control)  
+  control <- check.control.ode(control)
+  if ( control$backend == "CVODES" )
+    stop("Cannot use CVODES backend with bd.t")
+  
   cache <- make.cache.bd.ode(tree, sampling.f, unresolved)
 
   if ( is.null(names(functions)) && length(functions) == 2 )
@@ -26,8 +29,7 @@ make.bd.t <- function(tree, functions, sampling.f=NULL,
   n.args <- attr(pars.t, "n.args")
   is.constant.arg <- attr(pars.t, "is.constant.arg")
   
-  branches <- make.branches.bd.t(control$safe, control$tol,
-                                 control$eps)
+  branches <- make.branches.bd.t(cache, control)
   initial.conditions <-
     make.initial.conditions.t(initial.conditions.bd.ode)
   const <- lfactorial(length(tree$tip.label) - 1)
@@ -55,16 +57,10 @@ make.bd.t <- function(tree, functions, sampling.f=NULL,
   .NotYetImplemented()
 }
 
-make.branches.bd.t <- function(safe=FALSE, tol=1e-8, eps=0) {
-  RTOL <- ATOL <- tol
-  e <- new.env()
-
-  bd.t <- make.ode("derivs_bd_t", "diversitree", "initmod_bd_t",
-                       2, safe)
-  branches <- function(y, len, pars, t0)
-    t(bd.t(y, c(t0, t0+len), list(pars, e), rtol=RTOL,
-           atol=ATOL)[-1,-1])
-
-  make.branches(branches, 2, eps)
+make.branches.bd.t <- function(cache, control) {
+  neq <- 2L
+  np <- 2L # not used
+  comp.idx <- 2L
+  make.ode.branches.t("bd_t", "diversitree", neq, np, comp.idx,
+                      control)
 }
-
